//
// Mode 2 Demo
//
// by dancotter@optushome.com.au
//
// this demo shows how to display a tile map in mode 2
// and how to use rotation and scaling to do cool stuff.

// the map is made up of 156 different 8x8pixel characters (tiles)

//includes:
#include "..\..\lib\gba.h" //gameboy advance header file
#include "mapdata.h" //character map data (Screen data)
#include "chardata.h" //character data (char data)
#include "paldata.h" //pallette data  (colors)

#include "..\..\lib\rs.h" //rotation scaling header

void AGBMain() //start of main code
	{
      s16 zoom=0x100;//the zoom factor
      s16 angle = 0;//angle
      u16 way=0;//travel direction, since we will be zooming in and out
      			 //it will store the 'way' we are travelling (either in or out).


		//set video mode to mode2
      //mode 2 uses background 2 and 3 for rotation and scaling.
		mode2();

      //load the character data to bank 0
      //since there are 156 characters, each made up of 64 characters,
      //there are 11904 bytes of data to load.
      loadchardata(0, chardata, 14464);

      //load the screen data to vram
      //i picked block 13 for no particular reason...
      //since the screen map is 64 by 64 tiles (which is 4096 tiles)
      //we need to load 4096 half words
      //loadscreendata(13, mapdata, 1024);
      loadscreendatars(13,mapdata,4096);

      //load the background color palette
      set_pal(paldata, 0, 256);

      //set the background to 256x256
      rsbgsize512x512(&REG_BG2CNT);

      //set warp around on so when we zoom out the background repeates itself
      rsbgwraparound(&REG_BG2CNT);

      //set the screen base block to 13 (this is the block where we loaded
      //data to above)
      bgscrnbaseblock(&REG_BG2CNT,13);

      //same thing for character base block
      bgcharbaseblock(&REG_BG2CNT,0);

      //using multiple baseblocks we could store lots of maps and character
      //data in the vram and quickly switch between them with out having
      //to load from the cart.

      //set the background to 256 colors, 1 palette.
      bg256x1col(&REG_BG2CNT);

      forceblankoff();
      //turn on the screen

      //turn he background on:
      background2on();



      while(!0)
      //loop for ever
      	{
         //wait for vertical blank to start
         //dont up date graphics things while not in vertical blanck
         //unless what your using is turned off
         wait_vbl_start(1);

			//update the rotation/scaling properties.
         rs_bg2(angle,120,80,512<<8,512<<8,zoom,zoom);


         if(way==0) zoom+=10;//if way is 0 the increase zoom factoer
         else zoom-=10;//else decrease zoom factor
         if(zoom<=0x20) way=0;//if zoom is to low then change the way direction
         if(zoom>=0x800)way=1;//if zoom is to high then change the way direction
         angle++;//increase rotation angle
         if(angle==360) angle = 0;//only angles 0-359 are allowed,
      									 //so if angle is 360 reset it to zero.


         }
	}
