// Game Boy Advance
// Background and Objects Demo
//
// by dancotter@optushome.com.au
//
//this demo uses all 4 txt backgrounds of mode 0 to display a sea scene.
//the first 3 backgrounds are different layers of waves, thet last background
//is the backdrop with the sun and some clouds
//
//there are also objects, these are are sailing ship, some birds and some
//clouds (objects are also known as sprites).
//everything moves around and things bob up and down as well, the birds will
//also flap there wings by change which tile that are shown as

//includes:
#include "..\..\lib\gba.h" //gameboy advance header file

#include "paldata.h" //pallette data for all the backgrounds
#include "map1data.h"//background map data (screen data)
#include "ch1data.h"//background tile data (character data)
#include "map2data.h"
#include "ch2data.h"
#include "map3data.h"
#include "ch3data.h"
#include "map4data.h"
#include "ch4data.h"
#include "spritedat.h"//sprite data
#include "objpal.h"//sprite palette data

//the first color of each palette is transparent, in the case of the bitmaps
//for this demo, that color is black. So anything black will be transparent.

u8 test;

void AGBMain() //start of main code
	{

      u8 x[4] = {0,0,0,0};
      //x co-ord for scrolling of the 4 backgrounds

      u8 y[5] = {130,132,133,0};
      // y co-ords for scrolling of the backgrounds

      u16 sx[5] = {50,50,60,70,80};//the sprite x co-ords go to 512, not 256,
                      //so u8 can not be used or the sprite will "jump"
                      //onto the end of the screen as it travells along

      u8 sy[5] = {100,10,20,30,40};
      //sprite y co-ords

      u8 counter = 0;
      //counter used for counting the frames, it will
      //be used to scroll the backgrounds and sprites at different speeds.

      u8 i = 0;
      //loop counter

		//set video mode to mode0
		mode0();

      //load in the character data
      //background - number of characters
      //1 - 640
      //2 - 256
      //3 - 192
      //4 - 4160
      loadchardata(0, ch1data, 576);
      loadchardata(1, ch2data, 320);
      loadchardata(2, ch3data, 256);
      loadchardata(3, ch4data, 4160);

      //load the screen data to vram
      //i will use the screen blocks 7,15,23 and 31
      //they are not fulled with character data so its ok to use them.
      loadscreendatatxt(7, map1data, 1024);
      loadscreendatatxt(15, map2data, 1024);
      loadscreendatatxt(23, map3data, 1024);
      loadscreendatatxt(31, map4data, 1024);

      //load the background color palette
      set_pal(paldata, 0, 256);//background is 256 colors starting at 0
      set_pal(objpal, 256,256);//object is 256 colors starting at 256

      //set the backgrounds to 256x256
      //(this is not actually needed since this is the default size)
      txtbgsize256x256(&REG_BG0CNT);
      txtbgsize256x256(&REG_BG1CNT);
      txtbgsize256x256(&REG_BG2CNT);
      txtbgsize256x256(&REG_BG3CNT);

      //set the backgrounds to 256 colors, 1 palette.
      bg256x1col(&REG_BG0CNT);
      bg256x1col(&REG_BG1CNT);
      bg256x1col(&REG_BG2CNT);
      bg256x1col(&REG_BG3CNT);


      //set the screen base blocks for the backgrounds
      bgscrnbaseblock(&REG_BG0CNT,7);
      bgscrnbaseblock(&REG_BG1CNT,15);
      bgscrnbaseblock(&REG_BG2CNT,23);
      bgscrnbaseblock(&REG_BG3CNT,31);

      //same thing for character base block
      bgcharbaseblock(&REG_BG0CNT,0);
      bgcharbaseblock(&REG_BG1CNT,1);
      bgcharbaseblock(&REG_BG2CNT,2);
      bgcharbaseblock(&REG_BG3CNT,3);

      //set the background priorities
      bgpriority(&REG_BG0CNT,0);
      bgpriority(&REG_BG1CNT,1);
      bgpriority(&REG_BG2CNT,2);
      bgpriority(&REG_BG3CNT,3);

      //Now to set up the sprites (sprite 0 is the ship):

      //2d mapping format
      obj_2d_mapping();

      //load object character data
      loadobjchardata(spritedat,8192,0);

      set_obj_a0(0,OAM0_256COL);
      //set up the attribute 0, 256colors 1 palette

      set_obj_a1(0,0);
      //set up attribute 1, must set it to 0 because OAM is RAM and will
      //be a random value when the GBA is turned on.

      set_obj_a2(0,OAM2_PRIORITY2);
      //set up attribute 2, the priority is 2, so the first 2 backgrouds
      //are infront of it, and the other 2 are behind it.

      set_sprite_pos(0,sx[0],sy[0]);
      //move the sprite to position

      set_sprite_num(0,0);
      //set the sprites number for where tile data is taken from in obj char ram

      set_sprite_size64x32(0);
      //set the sprites size to 64 pixels by 32 pixels


      //the sprites are:
      //sprite 0 - ship (done above)
      //sprite 1 - bird
      //sprite 2 - small bird
      //sprite 3 - small cloud
      //sprite 4 - big cloud

      //have to set attributes for all the sprites

      for(i=1;i<5;i++)	{
      						set_obj_a0(i,OAM0_256COL);
      						set_obj_a1(i,0);
                        set_obj_a2(i,OAM2_PRIORITY2);
						      set_sprite_pos(i,sx[i],sy[i]);
                        //these attributes are same for all sprites
								}
      set_sprite_num(1,128);
      set_sprite_num(2,160);
      set_sprite_num(3,132);
      set_sprite_num(4,136);
      //they all have different tiles

      set_sprite_size8x8(1);
      set_sprite_size8x8(2);
      set_sprite_size16x16(3);
      set_sprite_size32x16(4);
      //they are different sizes

      forceblankoff();
      //turn on the screen

      wait_vbl_start(1);
      //wait for vertical blank to start
      //this avoids turning on the displays half way down the screen etc.

      //turn he backgrounds on:
      background0on();
      background1on();
      background2on();
      background3on();

      //turn the objects on
      objectson();


      while(!0)
      //loop for ever
      	{
         //increment counter
         counter++;

         //background 0 scrolls left 60 pixels per second
         x[0]++;//so increment x[0]
         sx[1]++;//moves sprite to left 60 pixels per second

    		set_sprite_num(1,130);//change the birds tile
         set_sprite_num(2,162);//change the birds tile

         //on ever second frame do:
         if((counter%2) == 0)
         		{
         		set_sprite_num(1,128);
			      set_sprite_num(2,160);
               sx[0]++;
               sx[2]--;
               }

         ///on every 4th 16th and 64th frame do:
         if((counter%4) == 0) {x[1]--; sx[3]++;}
         if((counter%16) == 0) {x[2]++; sx[4]--;}
         if((counter%64) == 0) x[3] ++;


         //change the y scroll offsets (vertical)
         //this will make the wave bob up and down
         if (counter%8 == 0)
         	switch (y[0])
            	{
               	case 130:
                  y[0]++;
                  break;
                  default:
                  y[0]--;
                  break;
               }
         if (counter%32 == 0)
         	switch (y[2])
            	{
               	case 133:
                  y[2]++;
                  break;
                  default:
                  y[2]--;
                  break;
               }

         //wait for vertical blank to start, there are roughly 60 per second.
         wait_vbl_start(1);

         //scroll the background
         scrollbg0(x[0],y[0]);
         scrollbg1(x[1],y[1]);
         scrollbg2(x[2],y[2]);
         scrollbg3(x[3],y[3]);

         for (i=0;i<5;i++) if (sx[i] == 241) sx[i] = 448;
         //if the spite goes out shift it acroos to the end so it can wrap
         //around quicker

         for (i=0;i<5;i++) if (sx[i] == 0xFFFF) sx[i] = 511;
         //if the x pos goes back from 0 bring it back to 511

         for (i=0;i<5;i++) if (sx[i] == 447) sx[i] = 240;
         //if the x pos is 447 bring it acroos to 240 to wrap around quicker

         for (i=0;i<5;i++) if (sx[i] == 512) sx[i] = 0;
         //if the x pos is 512 wrap back to zero.

         //move sprite
         for (i=0;i<5;i++) set_sprite_pos(i,sx[i],sy[i]);

         //just for fun if the player is pressing start, invert the sprite
         //colors... and if pressing select, invert the background colors
         if(pad(J_START)) {wait_vbl_start(1); inv_pal(256,256); wait_pad_up();}
         if(pad(J_SELECT)){wait_vbl_start(1); inv_pal(0,256);wait_pad_up();}

         }
	}


