//=========================================================
//            GAMEBOY ADVANCE - C LIBRARY
//=========================================================
//            dancotter@optushome.com.au
//
//Version 0.6.2 - 25 August 2001
//
// PLEASE SEE THE MANUAL.DOC file for information on the
// functions, it is no longer in this file.

//================================================================
//type definitions:
typedef unsigned char           u8;
typedef unsigned short          u16;
typedef unsigned long           u32;
typedef signed char             s8;
typedef signed short            s16;
typedef signed long             s32;
typedef const unsigned char     cu8;
typedef const unsigned short    cu16;
typedef const unsigned long     cu32;
typedef const signed char       cs8;
typedef const signed short      cs16;
typedef const signed long       cs32;
typedef volatile unsigned char  vu8;
typedef volatile unsigned short vu16;
typedef volatile unsigned long  vu32;
typedef volatile signed char    vs8;
typedef volatile signed short   vs16;
typedef volatile signed long    vs32;

//bits
#define BIT0    0x0001
#define BIT1    0x0002
#define BIT2    0x0004
#define BIT3    0x0008
#define BIT4    0x0010
#define BIT5    0x0020
#define BIT6    0x0040
#define BIT7    0x0080
#define BIT8    0x0100
#define BIT9    0x0200
#define BIT10   0x0400
#define BIT11   0x0800
#define BIT12   0x1000
#define BIT13   0x2000
#define BIT14   0x4000
#define BIT15   0x8000

//GBA Registers - DMA and Communication Registers are not included, yet
#define REG_WAITCNT     *(vu16*)0x04000204
#define REG_VCOUNT      *(vu16*)0x04000006
#define REG_DISPSTAT    *(vu16*)0x04000004
#define REG_DISPCNT     *(vu16*)0x04000000
#define REG_BG0CNT      *(vu16*)0x04000008
#define REG_BG1CNT      *(vu16*)0x0400000A
#define REG_BG2CNT      *(vu16*)0x0400000C
#define REG_BG3CNT      *(vu16*)0x0400000E
#define REG_MOSAIC      *(vu16*)0x0400004C
#define REG_BG2X_L      *(vu16*)0x04000028
#define REG_BG2X        *(vs32*)0x04000028
#define REG_BG3X_L      *(vu16*)0x04000038
#define REG_BG3X        *(vs32*)0x04000038
#define REG_BG2X_H      *(vu16*)0x0400002A
#define REG_BG3X_H      *(vu16*)0x0400003A
#define REG_BG2Y_L      *(vu16*)0x0400002C
#define REG_BG2Y        *(vs32*)0x0400002C
#define REG_BG3Y_L      *(vu16*)0x0400003C
#define REG_BG3Y        *(vs32*)0x0400003C
#define REG_BG2Y_H      *(vu16*)0x0400002E
#define REG_BG3Y_H      *(vu16*)0x0400003E
#define REG_BG2PA       *(vs16*)0x04000020
#define REG_BG3PA       *(vs16*)0x04000030
#define REG_BG2PB       *(vs16*)0x04000022
#define REG_BG3PB       *(vs16*)0x04000032
#define REG_BG2PC       *(vs16*)0x04000024
#define REG_BG3PC       *(vs16*)0x04000034
#define REG_BG2PD       *(vs16*)0x04000026
#define REG_BG3PD       *(vs16*)0x04000036
#define REG_BG0HOFS     *(vu16*)0x04000010
#define REG_BG1HOFS     *(vu16*)0x04000014
#define REG_BG2HOFS     *(vu16*)0x04000018
#define REG_BG3HOFS     *(vu16*)0x0400001C
#define REG_BG0VOFS     *(vu16*)0x04000012
#define REG_BG1VOFS     *(vu16*)0x04000016
#define REG_BG2VOFS     *(vu16*)0x0400001A
#define REG_BG3VOFS     *(vu16*)0x0400001E
#define REG_WIN0H       *(vu16*)0x04000040
#define REG_WIN1H       *(vu16*)0x04000042
#define REG_WIN0V       *(vu16*)0x04000044
#define REG_WIN1V       *(vu16*)0x04000046
#define REG_WININ       *(vu16*)0x04000048
#define REG_WINOUT      *(vu16*)0x0400004A
#define REG_BLDCNT      *(vu16*)0x04000050
#define REG_BLDALPHA    *(vu16*)0x04000052
#define REG_BLDY        *(vu16*)0x04000054
#define REG_SGFIFOA_L   *(vu16*)0x040000A0
#define REG_SGFIFOB_L   *(vu16*)0x040000A4
#define REG_SGFIFOA_H   *(vu16*)0x040000A2
#define REG_SGFIFOB_H   *(vu16*)0x040000A6
#define REG_SOUND1CNT_L *(vu16*)0x04000060
#define REG_SOUND1CNT_H *(vu16*)0x04000062
#define REG_SOUND1CNT_X *(vu16*)0x04000064
#define REG_SOUND2CNT_L *(vu16*)0x04000068
#define REG_SOUND2CNT_H *(vu16*)0x0400006C
#define REG_SG30_L      *(vu16*)0x04000070
#define REG_SG30_H      *(vu16*)0x04000072
#define REG_SOUND3CNT_X *(vu16*)0x04000074
#define REG_WAVE_RAM0_L *(vu16*)0x04000090
#define REG_WAVE_RAM0_H *(vu16*)0x04000092
#define REG_WAVE_RAM1_L *(vu16*)0x04000094
#define REG_WAVE_RAM1_H *(vu16*)0x04000096
#define REG_WAVE_RAM2_L *(vu16*)0x04000098
#define REG_WAVE_RAM2_H *(vu16*)0x0400009A
#define REG_WAVE_RAM3_L *(vu16*)0x0400009C
#define REG_WAVE_RAM3_H *(vu16*)0x0400009E
#define REG_SOUND4CNT_L *(vu16*)0x04000078
#define REG_SOUND4CNT_H *(vu16*)0x0400007C
#define REG_SOUNDCNT_L  *(vu16*)0x04000080
#define REG_SOUNTCNT_X  *(vu16*)0x04000084
#define REG_SOUNDCNT_H  *(vu16*)0x04000082
#define REG_SOUNDBIAS   *(vu16*)0x04000088
#define REG_TM0CNT_L    *(vu16*)0x04000100
#define REG_TM1CNT_L    *(vu16*)0x04000104
#define REG_TM2CNT_L    *(vu16*)0x04000108
#define REG_TM3CNT_L    *(vu16*)0x0400010C
#define REG_TM0CNT_H    *(vu16*)0x04000102
#define REG_TM1CNT_H    *(vu16*)0x04000106
#define REG_TM2CNT_H    *(vu16*)0x0400010A
#define REG_TM3CNT_H    *(vu16*)0x0400010E
#define REG_KEYINPUT    *(vu16*)0x04000130
#define REG_KEYCNT      *(vu16*)0x04000132
#define REG_IME         *(vu16*)0x04000208
#define REG_IE          *(vu16*)0x04000200
#define REG_IF          *(vu16*)0x04000202

//Memory locations
#define MEM_PALBG     *(u16*)0x05000000//BG Palette RAM start
#define MEM_PALOBJ    *(u16*)0x05000200//OBJ Palette RAM start
#define MEM_OBJDATAH  *(u16*)0x06014000//OBJ data high (bitmap modes)
#define MEM_OBJDATAL  *(u16*)0x06010000//OBJ data low area (tile modes)
#define MEM_OAM       *(u16*)0x07000000//OAM start
#define MEM_VBUFFER0  *(u16*)0x06000000//Video buffer 0 start (VRAM start)
#define MEM_VBUFFER1  *(u16*)0x0600A000//Video buffer 1 start
#define MEM_RAM_EXT   *(u32*)0x02000000//external ram start
#define MEM_RAM_INT   *(u32*)0x03000000//internal ram start

//character data base blocks
#define BB_char		 0x4000
#define BB_char0      *(u16*)0x06000000

//screen data base blocks
#define BB_screen     0x0800
#define BB_screen0	 *(u16*)0x06000000

//_________________________________________________________
//
//                       FUNCTIONS
//_________________________________________________________


//=========================================
//vbl.h - Vertical Blank functions
//=========================================

void wait_vbl(u16, u8);
void wait_vbl_start(u16);
void wait_vbl_end(u16);
void wait_vcount(u16);

//=========================================
//buffer.h - Video buffer functions
//=========================================

void framebuffer1(void);
void framebuffer0(void);
void switchbuffer(void);
u16 getframebuffer(void);

//=========================================
//dispcnt.h - Display Control functions
//=========================================

void forceblankoff(void);
void forceblankon(void);
void background0on(void);
void background1on(void);
void background2on(void);
void background3on(void);
void objectson(void);
void background0off(void);
void background1off(void);
void background2off(void);
void background3off(void);
void objectsoff(void);
void mode0(void);
void mode1(void);
void mode2(void);
void mode3(void);
void mode4(void);
void mode5(void);
void bg_mosaic_v_size(u16);
void bg_mosaic_h_size(u16);
void obj_mosaic_v_size(u16);
void obj_mosaic_h_size(u16);
void obj_1d_mapping(void);
void obj_2d_mapping(void);

//=========================================
//bgcnt.h - Background Control Functions
//=========================================

void txtbgsize256x256(vu16*);
void txtbgsize512x256(vu16*);
void txtbgsize256x512(vu16*);
void txtbgsize512x512(vu16*);
void rsbgsize128x128(vu16*);
void rsbgsize256x256(vu16*);
void rsbgsize512x512(vu16*);
void rsbgsize1024x1024(vu16*);
void bgscrnbaseblock(vu16*,u16);
void bgcharbaseblock(vu16*,u16);
void rsbgtransparent(vu16*);
void rsbgwraparound(vu16*);
void bg256x1col(vu16*);
void bg16x16col(vu16*);
void bgmosaicon(vu16*);
void bgmosaicoff(vu16*);
void bgpriority(vu16*,u16);
void scrollbg0(u16, u16);
void scrollbg1(u16, u16);
void scrollbg2(u16, u16);
void scrollbg3(u16, u16);

//=========================================
//m3.h - Mode 3 specific functions
//=========================================

void m3_loadbuffer(const u16 *);
void m3_put_pixel(u16 ,u16 ,u16 );
u16 m3_get_pixel(u16, u16);
void m4_loadbuffer(const u16 *, u8);
void m4_put_bg_2pixel(u16 , u16, u8, u8 , u8 );
u8 m4_get_bg_pixel(u16, u16, u8);
void m4_put_pixel(u16,u16,u8,u8);
void m5_loadbuffer(const u16 *, u8);
void m5_put_pixel(u16 ,u16 ,u16 ,u8 );
u16 m5_get_bg_pixel(u16 , u16 , u8 );

//=========================================
//pal.h - Palette functions
//=========================================

void pal_to(u16, u16, u8);
void bg_pal_to(u16 , u16);
void obj_pal_to(u16, u16);
void to_obj_pal(const u16 *, u16 );
void to_bg_pal(const u16 *, u16);
void to_pal(u16 , const u16 *, u8);
void inv_pal(u16, u16);
void set_pal_color(u16 ,u16 );
void set_pal(const u16 *, u16, u16);
void col_to(u16, u16 , u16 );

//=========================================
//joypad.h - Joypad functions
//=========================================

void wait_pad_up(void);
void wait_pad_down(void);
void wait_pad(u16 );
u16 pad(u16);

//=========================================
//interupt.h - Interrupt functions
//=========================================

void disableinterrupts(void);
void enableinterrupts(void);
void set_ie(u16);
void reset_if(u16);


//=========================================
//window.h - Window functions
//=========================================

void window0(u16, u16 , u16 , u16 );
void window1(u16, u16 , u16 , u16 );
void window0bg0outside (u16);
void window0bg1outside (u16);
void window0bg2outside (u16);
void window0bg3outside (u16);
void window0objoutside (u16);
void window1bg0inside (u16 );
void window1bg1inside (u16 );
void window1bg2inside (u16 );
void window1bg3inside (u16 );
void window1objinside (u16 );
void window0bg0inside (u16);
void window0bg1inside (u16);
void window0bg2inside (u16);
void window0bg3inside (u16);
void window0objinside (u16);
void window1bg0outside (u16 );
void window1bg1outside (u16 );
void window1bg2outside (u16 );
void window1bg3outside (u16 );
void window1objoutside (u16 );

//=========================================
//vram.h - Video RAM load functions, for all modes
//=========================================

void loadscreendatatxt(u16 ,cu16 *, u32 );
void loadscreendatars(u16 ,cu8 *, u32 );
void loadchardata(u16 , cu8 *, u32 );
void loadobjchardata(cu8 *, u32, u32);
void loadobjchardatah(cu8 *, u32, u32);
void loadchardata2b(u16 , cu8 *, u32);

//=========================================
//oam.h - object attribute memory header
//=========================================

void set_obj_a0 (u16, u16);
void set_obj_a1 (u16, u16);
void set_obj_a2 (u16, u16);
void set_sprite_pos(u16 , u16 , u16 );
void set_sprite_num(u16 , u16 );
void set_sprite_size8x8(u16 );
void set_sprite_size16x16(u16);
void set_sprite_size32x32(u16);
void set_sprite_size64x64(u16);
void set_sprite_size16x8(u16 );
void set_sprite_size32x8(u16 );
void set_sprite_size32x16(u16);
void set_sprite_size64x32(u16);
void set_sprite_size8x16(u16 );
void set_sprite_size8x32(u16 );
void set_sprite_size16x32(u16 );
void set_sprite_size32x64(u16 );

//=========================================
// includes
//=========================================

//function files
#include "dispcnt.h"
#include "colors.h"
#include "console.h"
#include "joypad.h"
#include "pal.h"
//rs.h, #include it individually
//rss.h or #include this to use just angles 0,1,2,3 as 0,90,180,270
#include "m3.h"
#include "m4.h"
#include "m5.h"
#include "buffer.h"
#include "vbl.h"
#include "interupt.h"
#include "bgcnt.h"
#include "window.h"
#include "vram.h"
#include "oam.h"
//eof
